//
// hello.mic.45.c
//
// electret microphone hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 4/1/14
// (c) Massachusetts Institute of Technology 2014
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay
#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_out (1 << PA1)			// solo TX
#define NPTS 100 // points in buffer


void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

int main(void) {
   //
   // main
   //
   static unsigned char i, array_lo[NPTS], array_hi[NPTS];
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
  // set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);
   //
   // init A/D
   //
   /* pag.134  datasheet */
    ADMUX |= (1 << MUX2) | (1 << MUX1) | (1 << MUX0); 	// devo usare ADC7 (PA7) quindi MUX5:0 = 000111. pag.145 datasheet
    
    /*
     *  REFS1 REFS0 == 0 0, VCC used as analog reference, disconnected from PA0 (AREF). 
     * !!!N.B.!!! DEVO fare cosi' perche' su PA0 ho il TX della seriale     
     * 
     * */
     
    ADMUX &= ~(1 << REFS1);
    ADMUX &= ~(1 << REFS0); 

    /*
     *   !!!! By default, the result is presented right adjusted, but can optionally be presented left adjusted by setting the * **ADLAR** bit in ADCSRB. !!!! 
     * If the result is left adjusted and no more than 8-bit precision is required, it is sufficient to read 
     * ADCH. Otherwise, ADCL must be read first, then ADCH, to ensure that the content of the data
     * registers belongs to the same conversion.
     * 
	 * */
//	ADCSRB |= (1 << ADLAR); 

	// **ADEN** : The ADC is enabled by setting the ADC Enable bit, *ADEN* in ADCSRA.
	/* **ADSC**
	 * The first conversion must be started by writing a logical one to the ADSC bit in ADCSRA. (p.135)
	 * 
	 * A single conversion is started by writing a logical one to the ADC Start Conversion bit, *ADSC*.
	 * This bit stays high as long as the conversion is in progress and will be cleared by hardware
	 * when the conversion is completed. 
	 * 
	 * If Auto Triggering is enabled, single conversions can be started by writing ADSC in ADCSRA to one. ADSC can also be used to determine if a conversion is in progress. The ADSC bit will be read as one during a conversion, independently of how the conversion was started.
	 * */
	 /*
	  * **ADSP2:0**
	  * These bits determine the division factor between the system clock frequency and the input clock to the ADC.
	  * 
	  * set prescaler at 1/64 because I want a sampling frequency of 125Khz and I have the Attiny at 8Mhz
	  * */
	 

	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADEN);
   
   //
   // main loop
   //
   while (1) {
      //
      // send framing
      //
      put_char(&serial_port, serial_pin_out, 1);
      char_delay();
      put_char(&serial_port, serial_pin_out, 2);
      char_delay();
      put_char(&serial_port, serial_pin_out, 3);
      char_delay();
      put_char(&serial_port, serial_pin_out, 4);
      char_delay();
      //
      // free-running sample loop
      //
      for (i = 0; i < NPTS; ++i) {
         //
         // initiate conversion
         //
         ADCSRA |= (1 << ADSC);
         //
         // wait for completion
         //
         while (ADCSRA & (1 << ADSC));
         //
         // save result
         //
         array_lo[i] = ADCL;
         array_hi[i] = ADCH;
         }
      for (i = 0; i < NPTS; ++i) {
         //
         // send result
         //
         put_char(&serial_port, serial_pin_out, array_lo[i]);
         put_char(&serial_port, serial_pin_out, array_hi[i]);
         }
      }
   }
